# tc_rmax_estimatefromR34kt.py

## Predict tropical cyclone Rmax from Vmax, R34kt, and latitude
### Source: Chavas D.R. and J. A.. Knaff (2022). A simple model for predicting the tropical cyclone radius of maximum wind from outer size. Wea. For., 37(5), pp.563-579
### https://doi.org/10.1175/WAF-D-21-0103.1

### 3 simple steps: Eq 3+4+7 from the paper
### Plus optional final bias adjustment: Eq. 8 (applied only for Rmax>60 km; this adjustment is only significant for larger Rmax > 60 km  -- see Fig 8 of paper)

### Cite this code: Daniel Robert Chavas (2022). Code file for Rmax prediction model and relevant data used in the analyses (Chavas and Knaff 2022, Weather and Forecasting). (Version 1.1). Purdue University Research Repository. doi:10.4231/WMMS-XY76

import numpy as np

def predict_Rmax_from_R34kt(
    VmaxNHC_ms: float,
    R34ktmean_km: float,
    lat: float
) -> float:
    """
    # Predict Rmax (km) from NHC point‐max wind speed (m/s),
    # R34kt mean radius (km), and latitude (deg), using CK22.
    # """
    
    R34ktmean_m = R34ktmean_km * 1000
    # Calculate M34kt (Eq 2 of CK22)
    omeg = 7.292e-5  # [s-1]
    fcor = 2 * omeg * np.sin(np.radians(np.abs(lat)))
    ms_kt = 0.5144444  # 1 kt = 0.514444 m/s
    V34kt = 34 * ms_kt  # [m/s]
    M34kt = R34ktmean_m * V34kt + 0.5 * np.abs(fcor) * R34ktmean_m**2
    
    # Define intermediate predictor
    halffcorR34kt_ms = 0.5 * fcor * R34ktmean_m
    
    # Estimate Mmax/M34kt (Eq 7 of CK22)
    # Model: Z = b*exp(beta_Vmax*(Vmax-V34kt)+beta_VfR*(Vmax-V34kt)*(0.5*f*R34kt))
    # Coefficients estimated from EBTK 2004-2020 (Eq 7 / Table 2 of CK22)
    coefs = {
        "b": 0.699,  # b
        "c": -0.00618,  # beta_Vmax
        "f": -0.00210  # beta_VfR
    }
    
    MmaxM34kt_predict = coefs["b"] * np.exp(
        coefs["c"] * (VmaxNHC_ms - V34kt) + coefs["f"] * (VmaxNHC_ms - V34kt) * halffcorR34kt_ms
    )
    
    # Solve for predicted Rmax (Mmax = Rmax*Vmax + 0.5*fcor*Rmax^2)
    # (Eq 3 of CK22)
    Mmax_predict = MmaxM34kt_predict * M34kt  # Mmax = MmaxM34kt * M34kt
    
    # (Eq 4 of CK22)
    Rmax_predict = (VmaxNHC_ms / fcor) * (
        np.sqrt(1 + (2 * fcor * Mmax_predict / (VmaxNHC_ms**2))) - 1
    )
    
    Rmax_predict_km = Rmax_predict / 1000
    km_nautmi = 1.852
    Rmax_predict_nautmi = Rmax_predict_km / km_nautmi
    # print("VmaxNHC_ms =", VmaxNHC_ms,'m/s')
    # print("R34ktmean_km =", R34ktmean_m/1000,'km =', R34ktmean_m/1000 / km_nautmi,'naut mi')
    # print('Rmax_predict_km=', Rmax_predict_km,'km =', Rmax_predict_nautmi,'naut mi')
    
    ## Apply optional final bias correction (Eq.)
    ## Do only if Rmax > 60 km; below that the uncertainties are same order as the value itself
    ## Also, for very small values (<9 km) it could give a negative Rmax which makes no sense
    if Rmax_predict_km > 60:
      Rmax_predict_km_biasadjgt60km = (1/0.76)*(Rmax_predict_km-9.02)
      Rmax_predict_nautmi_biasadjgt60km = Rmax_predict_km_biasadjgt60km / km_nautmi
      # print('Rmax_predict_km_biasadjgt60km=', Rmax_predict_km_biasadjgt60km,' km =', Rmax_predict_nautmi_biasadjgt60km,' naut mi')
      # print('Rmax estimation from R34kt: final bias adjustment applied for Rmax>60km, based on CK22')
    # else:
      # print('Rmax estimation from R34kt: no final bias adjustment (only done for Rmax>60km)')

    print("Returning estimated radius of maximum wind [km]")
    return Rmax_predict_km



# windprofile.py

## Create a fast and robust radial profile of the tropical cyclone rotating wind from inputs Vmax, R34kt, Rmax, and latitude.

#### This code uses a modified‐Rankine vortex between Rmax and R34kt and the E04 model beyond R34kt (and a quadratic profile inside the eye). It is very similar to the full physics-based wind profile model of Chavas et al. (2015) ([code here](http://doi.org/10.4231/CZ4P-D448)), but is simpler and much faster.

#### It is designed to guarantee that the profile fits both Rmax and R34kt and will be very close to the true outer radius (R0) as estimated by the full E04 outer solution. Hence, it is very firmly grounded in the known physics of the tropical cyclone wind field while also matching the input data. It is also guaranteed to be very well‐behaved for basically any input parameter combination.

#### Model basis:
#### Modified Rankine profile between Rmax and R34kt was shown to compare very well against high-quality subset of Atlantic Best Track database -- see Fig 8 of [Klotzbach et al. (2022, JGR-A)](https://doi.org/10.1029/2022JD037030)
#### Physics-based non-convecting wind field profile beyond R34kt was shown to compare very well against entire QuikSCAT database -- see Fig 6 of [Chavas et al. (2015, JAS)](https://doi.org/10.1175/JAS-D-15-0014.1)
#### Quadratic in the eye (U-shape is common)

from tcwindprofile.tc_outer_radius_estimate import estimate_outer_radius
from tcwindprofile.tc_outer_windprofile import outer_windprofile


def generate_wind_profile(Vmaxmean_ms, Rmax_km, R34ktmean_km, lat, plot=False):
    

    import numpy as np
    import math
    import matplotlib.pyplot as plt
    from scipy.interpolate import interp1d
    
    # Set default plot properties: font size 12 (half of 24) and default font
    plt.rcParams.update({
        'font.size': 12,
        'lines.linewidth': 2
    })
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ### INPUTS: AZIMUTHAL-MEAN VALUES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    # If you have VmaxNHC (i.e. Best Track values): Vmaxmean_ms = VmaxNHC_ms - 0.55 * Vtrans_ms (Eq 2 of CKK25 -- simple method to estimate azimuthal-mean Vmax from NHC point-max; factor originally from Lin and Chavas (2012))
    # If you have R34ktNHCquadmax (i.e. Best Track values): R34ktmean_m = 0.85 * fac_R34ktNHCquadmax2mean (Eq 1 of CKK25 -- simple estimate of mean R34kt radius from NHC R34kt (which is maximum radius of 34kt); factor originally from DeMaria et al. (2009))
    
    #Vmaxmean_ms = 45.8            # [m/s]; default 45.8; AZIMUTHAL-MEAN maximum wind speed
    #Rmax_m = 38.1 * 1000          # [m]; default 38.1*1000; from bias-adjusted CK22
    #R34ktmean_m = 228.3 * 1000    # [m]; default R34kt = 228.3*1000; MEAN radius of 34kt wind speed
    #lat = 20                      # [degN]; default 20N; storm-center latitude;
    
    Rmax_m = Rmax_km * 1000
    R34ktmean_m = R34ktmean_km * 1000
    ## Default values: Vmaxmean_ms = 45.8 m/s, Rmax_m = 38.1 km, R34ktmean_m = 228.3 km, lat = 20 --> R0=1174.6 km (sanity check)
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ### CALCULATE WIND PROFILE
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    # print("This code uses a modified‐Rankine vortex between Rmax and R34ktmean_m (default R34kt) and the E04 model beyond R34ktmean_m (and a quadratic profile inside the eye).")
    # print("It is designed to guarantee that the profile fits both Rmax and R34kt and will be very close to the true outer radius (R0) as estimated by the full E04 outer solution.")
    # print("It is also guaranteed to be very well‐behaved for basically any input parameter combination.")
    
    #%% Calculate some params
    ms_kt = 0.5144444             # 1 kt = 0.514444 m/s
    V34kt_ms = 34 * ms_kt            # [m/s]; outermost radius to calculate profile
    km_nautmi = 1.852
    omeg = 7.292e-5  # Earth's rotation rate
    fcor = 2 * omeg * math.sin(math.radians(abs(lat)))  # [s^-1]
    
    #%% Initial radius vector
    dr = 100  # [m]
    R0mean_initialtoolarge = 3000 * 1000
    rr_full = np.arange(0, R0mean_initialtoolarge + dr, dr)  # [m]
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%% STEP 1) Outer region wind profile (r>R34kt)
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%% STEP 1a) Very good estimate of outer edge of storm, R0mean (where v=0)
    #%% Analytic approximation of R0mean, from physical model of non-convecting wind profile (Emanuel 2004; Chavas et al. 2015 JAS)
    
    Cd = 1.5e-3  # [-]
    w_cool = 2 / 1000  # [m/s]
    beta = 1.35
    R0mean_dMdrcnstmod = estimate_outer_radius(
    R34ktmean_m=R34ktmean_m,
    V34kt_ms=V34kt_ms,
    fcor=fcor,
    Cd=Cd,
    w_cool=w_cool,
    beta=beta
    )
    
    R0_km = R0mean_dMdrcnstmod / 1000
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%% STEP 1b) Wind profile from R0mean --> R34kt
    #%% Exact solution to model of non-convecting wind profile (Emanuel 2004; Chavas et al. 2015 JAS)
    #%% Notes:
    #%% - It is a single integration inwards from R0. *Cannot* integrate out
    #%%   from R34kt, it may blow up (math is weird)
    #%% - Could do a simpler approximate profile instead, but this solution is
    #%%   just as fast doing some sort of curve fit, so might as well use exact
 
    # Returns:
    #   rr_full : radii [m]
    #   vv : wind speeds [m/s]
    rr_outer_m, vv_outer_ms = outer_windprofile(
    r0_m=R0mean_dMdrcnstmod,     # or your computed R0mean_dMdrcnstmod
    fcor=fcor,
    Cd=Cd,
    w_cool=w_cool,
    V34kt_ms=V34kt_ms,
    )
    
    # Interpolate to original radius vector, chop off NaNs beyond outer edge (which will extend a bit beyond r0)
    r0_plot = np.max(rr_outer_m)
    rr_full=rr_full[rr_full<=r0_plot]
    vv_E04approx = np.interp(rr_full, rr_outer_m, vv_outer_ms)

    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%% STEP 2) Inner region wind profile (r<=R34kt)
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    #%% Simple modified Rankine profile between R34kt and Rmax
    #%% With quadratic profile inside of Rmax
    alp_outer = np.log(Vmaxmean_ms / V34kt_ms) / np.log(Rmax_m / R34ktmean_m)
    vv_outsideRmax = np.full(rr_full.shape, np.nan)
    rr_outer_mask = rr_full > Rmax_m
    vv_outsideRmax[rr_outer_mask] = Vmaxmean_ms * (rr_full[rr_outer_mask] / Rmax_m)**alp_outer
    
    alp_eye = 2  # quadratic profile in eye
    vv_eye = Vmaxmean_ms * (rr_full / Rmax_m)**alp_eye
    vv_MR = np.concatenate((vv_eye[rr_full <= Rmax_m], vv_outsideRmax[rr_full > Rmax_m]))
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%% STEP 3) Merge inner and outer wind profiles
    #%% Simple exponential smoother of (inner-outer) moving outwards from R34kt
    
    # No smoothing: direct merge at R34ktmean_m -- will *not* match dV/dr without smoothing
    vv_MR_E04_R34ktmean_nosmooth = np.concatenate((vv_MR[rr_full <= R34ktmean_m], vv_E04approx[rr_full > R34ktmean_m]))
    
    # Match dV/dr at R34kt: exponential smoother of (inner-outer) moving outwards from R34kt
    ii_temp = rr_full > R34ktmean_m
    rr_beyondR34ktmean = rr_full[ii_temp]
    vv_MR_beyondR34ktmean = vv_MR[ii_temp]
    vv_E04approx_beyondR34ktmean = vv_E04approx[ii_temp]
    R_transitionouteredge = R34ktmean_m + (R0_km*1000 - R34ktmean_m)* (1 / 2)   #1/2 of the distance from R34kt to R0
    def taper_cubic(r, Rin, Rout):
        """1 at Rin, 0 at Rout, derivative continuous at both, smooth cubic Hermite."""
        s = (r - Rin) / (Rout - Rin)
        s = np.clip(s, 0, 1)
        return 1 - (3*s*s - 2*s*s*s)
        #note: tried many other more complex options, but they only create additional "knees" in the curve that make things worse
        #tried: quintic (not better), cosine (nearly identical), cubic with s^p p<1 (sharper transition but now derivative may be discontinuous at Rin), beta with sharpness param (maintains derivative continuity, but is just bumpier)
    w = taper_cubic(rr_beyondR34ktmean, R34ktmean_m, R_transitionouteredge)

    vv_E04approx_adj = vv_MR_beyondR34ktmean * w + vv_E04approx_beyondR34ktmean * (1 - w)
    vv_MR_E04_R34ktmean = np.concatenate((vv_MR[rr_full <= R34ktmean_m], vv_E04approx_adj))
    
    # Interpolate to 1 km resolution
    rr_km = rr_full / 1000
    rr_km_interp = np.arange(0, rr_km.max(), 0.1)  # 0.1 km resolution
    vv_ms_interp = np.interp(rr_km_interp, rr_km, vv_MR_E04_R34ktmean)



    if plot:
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        #%% Make plot
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        # Figure dimensions: original 30 cm x 30 cm, now half: 15 cm x 15 cm (converted to inches)
        fig, ax = plt.subplots(figsize=(15/2.54, 15/2.54))
        ax.plot(rr_km_interp, vv_ms_interp, 'm-', linewidth=3)
        ax.plot(rr_full / 1000, vv_MR_E04_R34ktmean_nosmooth, 'g:', linewidth=3)   #compare no smooth vs. smoothed
        ax.plot(Rmax_m / 1000, Vmaxmean_ms, 'k.', markersize=20)
        ax.plot(R34ktmean_m / 1000, V34kt_ms, 'k.', markersize=20)
        ax.plot(R0_km, 0, 'm.', markersize=20)
        ax.set_xlabel('radius [km]')
        ax.set_ylabel('azimuthal wind speed [m/s]')
        ax.axis([0, 1.1 * R0mean_dMdrcnstmod / 1000, 0, 1.1 * Vmaxmean_ms])
        # ax.axis([0, 600, 0, 1.1 * Vmaxmean_ms])
        ax.set_title('Complete wind profile', fontsize=12)
        
        # Annotate the top right corner with "Inputs:" and the values of Vmaxmean_ms, Rmax_m, (R34ktmean_m, V34kt_ms), and lat
        annotation = (f"Inputs:\n"
                      f"Vmax_mean = {Vmaxmean_ms:.1f} m/s\n"
                      f"Rmax = {Rmax_m/1000:.1f} km\n"
                      f"(R34kt_mean, V34kt_ms) = ({R34ktmean_m/1000:.1f} km, {V34kt_ms:.1f} m/s)\n"
                      f"lat = {lat:.1f}°N\n"
                      f"\n"
                      f"Output:\n"
                      f"R0_mean = {R0mean_dMdrcnstmod / 1000:.1f} km")
        ax.text(0.95, 0.95, annotation, transform=ax.transAxes, ha='right', va='top',
                fontsize=10, bbox=dict(facecolor='white', edgecolor='none', alpha=0.7))
        
        plt.savefig('windprofile.jpg', format='jpeg')
        plt.show()
        print("Made plot of wind profile!")
               
    # Return radius [km], wind speed [m/s], true R0 [km], estimated R0 [km]
    print("Returning radius vector [km], wind speed vector [m/s], estimated outer radius [km]")
    return rr_km_interp, vv_ms_interp, R0_km


    
    
if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("--vmax_ms", type=float, default=45.8, help="Vmax in m/s")
    parser.add_argument("--rmax_km", type=float, default=38.1, help="Rmax in km")
    parser.add_argument("--r34_km", type=float, default=228.3, help="R34ktmean in km")
    parser.add_argument("--lat", type=float, default=20, help="Latitude in degrees")
    args = parser.parse_args()

    # Convert km to m
    rr_km, vv_ms, R0_km = generate_wind_profile(args.vmax_ms, args.rmax_km, args.r34_km, args.lat)




# tc_pmin_estimatefromR34kt.py

## Predict tropical cyclone Pmin from Vmax, R34kt, latitude, translation speed, and environmental pressure (or pressure of outermost closed isobar)
### Source: Chavas D.R., Knaff J.A. and P. Klotzbach  (2025). A Simple Model for Predicting Tropical Cyclone Minimum Central Pressure from Intensity and Size. Wea. For., 40(2), pp.333-346
### https://doi.org/10.1175/WAF-D-24-0031.1
### Simple: Eq 5 to estimate dP; use Penv (or Eq 6 from Poci) to estimate Pmin.

### Cite this code: Chavas, D. R. (2025). Chavas Knaff Klotzbach 2024 Pmin prediction model. Purdue University Research Repository. doi:10.4231/NKX4-MM81

import numpy as np


def predict_Pmin_from_R34kt(
    VmaxNHC_ms: float,
    R34ktmean_km: float,
    lat: float,
    Vtrans_ms: float,
    Penv_mb: float
) -> float:
    
    """
    # Predict Pmin (mb) from NHC point‐max wind speed (m/s), translation speed (m/s),
    # R34kt mean radius (km), latitude (deg), and environmental pressure (mb)
    # following CKK25
    # """

    # Calculate final predictors
    R34ktmean_m = R34ktmean_km * 1000
    Vmaxmean_ms = VmaxNHC_ms - 0.55 * Vtrans_ms  #Eq 2 of CKK25 -- simple method to estimate azimuthal-mean Vmax from NHC point-max; factor originally from Lin and Chavas (2012)
    omeg = 7.292e-5  # [s-1]
    fcor = 2 * omeg * np.sin(np.radians(np.abs(lat)))
    
    # Define intermediate predictor
    halffcorR34kt_ms = 0.5 * fcor * R34ktmean_m
    
    # Eq. 5 of CKK25: Best fit model coefficients
    coefs = np.array([-6.60, -0.0127, -5.506, 109.013])
    dP_predict_mb = (
        coefs[0] + coefs[1] * Vmaxmean_ms**2 +
        coefs[2] * halffcorR34kt_ms +
        coefs[3] * halffcorR34kt_ms / Vmaxmean_ms
    )
    
    Pmin_predict_mb = Penv_mb + dP_predict_mb   #Eq 3 of CKK25
    # print("Vmaxmean_ms =", Vmaxmean_ms,' m/s')
    # print("R34ktmean_km =", R34ktmean_m/1000,'km =', R34ktmean_m/1000 / km_nautmi,'naut mi')
    # print("dP_predict_mb =", dP_predict_mb,' mb')
    # print("Pmin_predict_mb =", Pmin_predict_mb,' mb (Penv =', Penv_mb,' mb)')

    print("Returning minimum central sea-level pressure [mb]")
    return Pmin_predict_mb, dP_predict_mb


# tcwindprofile/pressure_profile.py
# Calculate the pressure profile from an input wind profile that matches the input minimum pressure

# Integrates the gradient wind balance equation, assumes density is constant.
# Allowing density to vary with pressure is also doable (since P decreases moving towards the center),
# but the math works out to be the same as simply rescaling the constant-density solution by a constant
# Default density value is 1.15, but exact value does NOT matter anyways since profile
# is rescaled by a constant in the end to match Pmin

import numpy as np
from scipy.interpolate import interp1d
import matplotlib.pyplot as plt

def pressure_profile_calcfromwindprofile(
    rr_km: np.ndarray,
    vv_ms: np.ndarray,
    Renv_km: float,
    Penv_mb: float,
    Pmin_mb: float,
    lat: float,
    rho_kgm3: float = 1.15,  # [kg/m³], default constant density
    plot=False
) -> tuple[float, np.ndarray]:
    """
    Estimate minimum pressure using gradient wind balance (GWB).
    
    Parameters
    ----------
    rr_km : ndarray
        Radius [m]
    vv_ms : ndarray
        Azimuthal wind speed [m/s]
    Renv_km : float
        Environmental radius where Penv applies [m]
    Penv_mb : float
        Environmental pressure [mb/hPa]
    Pmin_mb : float
        Minimum central pressure [mb/hPa]
    lat : float
        Latitude [deg]
    rho_kgm3 : float
        Air density (constant) [kg/m³]

    Returns
    -------
    Pres_mb : ndarray
        Full pressure profile [mb]
    """
    # Check input range
    if np.nanmax(rr_km[~np.isnan(vv_ms)]) < Renv_km:
        raise ValueError("Renv lies beyond the input radial profile")

    # Unit conversion to MKS
    rr_m = rr_km * 1000 #[m]
    Renv_m = Renv_km * 1000 #[m]
    
    # Constants
    omega = 7.292e-5  # [s^-1]
    fcor = 2 * omega * np.sin(np.radians(abs(lat)))

    # High-resolution radius grid
    rr_hr = np.arange(0, Renv_m + 10, 10)  # [m]

    # Interpolate V to high-res grid
    vv_hr = interp1d(rr_m, vv_ms, bounds_error=False, fill_value=np.nan)(rr_hr)

    # Compute dP/dr via gradient wind balance
    term1 = fcor * vv_hr
    with np.errstate(divide='ignore', invalid='ignore'):
        term2 = np.where(rr_hr>0, vv_hr**2/rr_hr, 0.0)

    # term2 = vv_hr**2 / rr_hr
    dPdr = rho_kgm3 * (term1 + term2)

    # Integrate inward to get pressure deficit (dP defined POSITIVE)
    dr = np.nanmean(np.diff(rr_hr))
    dP_hr = np.flip(np.nancumsum(np.flip(dPdr * dr)))

    # Linear extrapolation for first point (if needed)
    if len(dP_hr) >= 3:
        dP_hr[0] = dP_hr[1] + (dP_hr[1] - dP_hr[2])

    # Interpolate back to original radius vector
    dP_interp = interp1d(rr_hr, dP_hr, bounds_error=False, fill_value=np.nan)
    dP_mb = dP_interp(rr_m) / 100  # convert Pa → mb (a.k.a. hPa)

    # Rescale pressure deficit by constant to match input dP
    dP_center_mb = Penv_mb - Pmin_mb  #dP defined POSITIVE
    dP_center_prof_mb = np.nanmax(dP_mb)  #dP defined POSITIVE
    # print(dP_center_prof_mb)
    dP_mb = dP_mb*(dP_center_mb/dP_center_prof_mb)
    
    # Compute full pressure profile
    pp_mb = Penv_mb - dP_mb
    


    if plot:
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        #%% Make plot
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        # Figure dimensions: original 30 cm x 30 cm, now half: 15 cm x 15 cm (converted to inches)
        fig, ax = plt.subplots(figsize=(15/2.54, 15/2.54))
        ax.plot(rr_m / 1000, pp_mb, 'm-', linewidth=3)
        ax.plot(0, Pmin_mb, 'k.', markersize=15)
        ax.plot([Renv_km, Renv_km], [0, 10*Penv_mb], 'k--', linewidth=1)
        ax.plot(Renv_km, Penv_mb, 'k.', markersize=15)
        ax.set_xlabel('radius [km]')
        ax.set_ylabel('sea-level pressure [mb]')
        ax.set_title('Complete pressure profile', fontsize=12)
        ax.axis([0, 1.1 * Renv_km, 0.98 * Pmin_mb, 1020])
        
        # Annotate the top right corner with
        annotation = (f"Pmin = {Pmin_mb:.1f} mb\n"
                      f"Penv = {Penv_mb:.1f} mb")
        ax.text(0.1, 0.2, annotation, transform=ax.transAxes, ha='left', va='top',
                fontsize=10, bbox=dict(facecolor='white', edgecolor='none', alpha=0.7))
        

        plt.savefig('pressureprofile.jpg', format='jpeg')
        plt.show()
        print("Made plot of pressure profile from input wind profile!")

    # Return pressure profile
    print("Returning sea-level pressure vector [mb] along input radius vector [km]")
    return pp_mb



# windprofile.py

## Create a fast and robust radial profile of the tropical cyclone rotating wind from inputs Vmax, R34kt, Rmax, and latitude.

#### This code uses a modified‐Rankine vortex between Rmax and R34kt and the E04 model beyond R34kt (and a quadratic profile inside the eye). It is very similar to the full physics-based wind profile model of Chavas et al. (2015) ([code here](http://doi.org/10.4231/CZ4P-D448)), but is simpler and much faster.

#### It is designed to guarantee that the profile fits both Rmax and R34kt and will be very close to the true outer radius (R0) as estimated by the full E04 outer solution. Hence, it is very firmly grounded in the known physics of the tropical cyclone wind field while also matching the input data. It is also guaranteed to be very well‐behaved for basically any input parameter combination.

#### Model basis:
#### Modified Rankine profile between Rmax and R34kt was shown to compare very well against high-quality subset of Atlantic Best Track database -- see Fig 8 of [Klotzbach et al. (2022, JGR-A)](https://doi.org/10.1029/2022JD037030)
#### Physics-based non-convecting wind field profile beyond R34kt was shown to compare very well against entire QuikSCAT database -- see Fig 6 of [Chavas et al. (2015, JAS)](https://doi.org/10.1175/JAS-D-15-0014.1)
#### Quadratic in the eye (U-shape is common)

# tcwindprofile/full_profile.py

def run_full_wind_model(
    VmaxNHC_kt: float,
    Vtrans_kt: float,
    R34kt_quad_max_nautmi: float,
    lat: float,
    Penv_mb: float,
    plot: bool = False
):
    """
    Full modeling pipeline:
    - Estimate Rmax from R34kt: ref Chavas and Knaff 2022 WAF)
    - Estimate R0 from R34kt: approximate version of outer model ref Emanuel 2004 / Chavas et al. 2015 JAS / Chavas and Lin 2016 JAS
    - Generate wind profile: merge simple inner + outer models, ref Klotzbach et al. 2022 JGR-A / Chavas and Lin 2016 JAS
    - Estimate Pmin: ref Chavas Knaff Klotzbach 2025 WAF
    - Generate pressure profile that matches Pmin: ref Chavas Knaff Klotzbach 2025 WAF
    """
    # Convert input units
    ms_per_kt = 0.5144444
    km_per_nautmi = 1.852
    VmaxNHC_ms = VmaxNHC_kt * ms_per_kt
    Vtrans_ms = Vtrans_kt * ms_per_kt
    Vmaxmean_ms = VmaxNHC_ms - 0.55 * Vtrans_ms  #Lin and Chavas (2012), Chavas Knaff Klotzbach (2025)
    R34ktmean_km = 0.85 * R34kt_quad_max_nautmi * km_per_nautmi
    
    V34kt_ms = 17.49
    
    if Vmaxmean_ms < V34kt_ms:
        raise ValueError("Vmaxmean_ms cannot be < 34 kt")

    ###########################
    # 1) Estimate Rmax (CK22)
    from tcwindprofile.tc_rmax_estimatefromR34kt import predict_Rmax_from_R34kt
    
    Rmax_km = predict_Rmax_from_R34kt(
        VmaxNHC_ms=VmaxNHC_ms,
        R34ktmean_km=R34ktmean_km,
        lat=lat
    )

    ###########################
    # 2) Estimate wind profile + R0 (Approximation to CLE15 model)
    # print(Vmaxmean_ms,Rmax_km,R34ktmean_km,lat,plot)
    from tcwindprofile.windprofile import generate_wind_profile
    # from tcwindprofile.tc_outer_radius_estimate import estimate_outer_radius
    # from tcwindprofile.tc_outer_windprofile import outer_windprofile
    
    rr_km, vv_ms, R0_km = generate_wind_profile(
        Vmaxmean_ms=Vmaxmean_ms,
        Rmax_km=Rmax_km,
        R34ktmean_km=R34ktmean_km,
        lat=lat,
        # plot=True
        plot=False
    )

    ###########################
    # 3) Estimate Pmin (CKK25)
    # print(VmaxNHC_ms,R34ktmean_km,lat,Vtrans_ms,Penv_mb)
    from tcwindprofile.tc_pmin_estimatefromR34kt import predict_Pmin_from_R34kt
    
    Pmin_estimate_mb, dP_estimate_mb = predict_Pmin_from_R34kt(
        VmaxNHC_ms=VmaxNHC_ms,
        R34ktmean_km=R34ktmean_km,
        lat=lat,
        Vtrans_ms=Vtrans_ms,
        Penv_mb=Penv_mb
    )

    ###########################
    # 4) Calculate pressure profile from wind field and matches Pmin
    from tcwindprofile.pressure_profile import pressure_profile_calcfromwindprofile                        # [deg]
    
    pp_mb = pressure_profile_calcfromwindprofile(
        rr_km=rr_km,
        vv_ms=vv_ms,
        Renv_km=R0_km,
        Penv_mb=Penv_mb,
        Pmin_mb=Pmin_estimate_mb,
        lat=lat,
        plot=False)
    
    ###########################
    # 5) Return final results
    return {
        "rr_km": rr_km,
        "vv_ms": vv_ms,
        "pp_mb": pp_mb,
        "Vmaxmean_ms": Vmaxmean_ms,
        "Rmax_km": Rmax_km,
        "V34kt_ms": V34kt_ms,
        "R34ktmean_km": R34ktmean_km,
        "R0_km": R0_km,
        "lat": lat,
        "Penv_mb": Penv_mb,
        "Pmin_mb": Pmin_estimate_mb,
    }



# tcwindprofile/plot_windprofile.py

import matplotlib.pyplot as plt

def plot_wind_and_pressure(
    # wind profile inputs
    rr_wind_km, vv_ms,
    Rmax_km, Vmaxmean_ms,
    R34kt_km, V34kt_ms,
    R0_km, lat_deg,
    # pressure profile inputs
    rr_pres_km, PP_mb,
    Renv_km, Penv_mb, Pmin_mb,
    save_path: str = "tc_wind_pressure_profiles"
):
    """
    Plot wind profile (top) and pressure profile (bottom) in one figure.

    Parameters
    ----------
    rr_wind_km : array_like
        Radii for wind [km]
    vv_ms : array_like
        Wind speeds [m/s]
    Rmax_km, Vmaxmean_ms : float
        Radius and value of maximum wind
    R34kt_km, V34kt_ms : float
        Radius and value of 34 kt mean wind
    R0_km : float
        Outer radius where wind drops to zero
    lat_deg : float
        Latitude (for annotation only)

    rr_pres_km : array_like
        Radii for pressure [km]
    PP_mb : array_like
        Pressure profile [mb]
    Renv_km, Penv_mb : float
        Environmental radius & pressure
    Pmin_mb : float
        Minimum central pressure

    save_path : str, optional
        If given, figure will be saved to `<save_path>.jpg`.
    
    Returns
    -------
    fig, (ax1, ax2)
        The Figure and Axes objects.
    """

    # Figure size: 15 cm wide, 30 cm tall
    # fig, (ax1, ax2) = plt.subplots(
    #     2, 1,
    #     figsize=(15/2.54, 30/2.54),
    #     sharex=False
    # )
    fig, (ax1, ax2) = plt.subplots(
        2, 1,
        figsize=(15/2.54, 22.5/2.54),
        sharex=False,
        gridspec_kw={'height_ratios': [2, 1]}
    )

    # ── Top: Wind profile ───────────────────────────────────────────────
    ax1.plot(rr_wind_km, vv_ms, color='m', linewidth=3, zorder=1)
    ax1.plot(Rmax_km, Vmaxmean_ms, 'k.', markersize=20, zorder=5)
    ax1.plot(R34kt_km, V34kt_ms, 'k.', markersize=20, zorder=5)
    ax1.plot(R0_km, 0, 'm*', markersize=20, zorder=5)
    ax1.plot([R0_km, R0_km], [0, 1000], 'k--', linewidth=1)
    ax1.set_xlim(0, 1.1 * R0_km)
    ax1.set_ylim(0, 1.1 * Vmaxmean_ms)
    ax1.set_xlabel('radius [km]')
    ax1.set_ylabel('azimuthal wind speed [m/s]')
    ax1.set_title('Wind profile', fontsize=12)

    ann1 = (
        f"Inputs:\n"
        f"Vmax_mean = {Vmaxmean_ms:.1f} m/s\n"
        f"Rmax = {Rmax_km:.1f} km\n"
        f"R34kt_mean = {R34kt_km:.1f} km\n"
        f"lat = {lat_deg:.1f}°N\n\n"
        f"Output:\n"
        f"R0 = {R0_km:.1f} km"
    )
    ax1.text(
        0.95, 0.95, ann1,
        transform=ax1.transAxes,
        ha='right', va='top',
        fontsize=10,
        bbox=dict(facecolor='white', edgecolor='none', alpha=0.7)
    )

    # ── Bottom: Pressure profile ────────────────────────────────────────
    ax2.plot(rr_pres_km, PP_mb, color='m', linewidth=3, zorder=1)
    ax2.plot(0, Pmin_mb, 'k.', markersize=15, zorder=5)
    ax2.plot([Renv_km, Renv_km], [0, 10000], 'k--', linewidth=1)
    ax2.plot(Renv_km, Penv_mb, 'k.', markersize=15, zorder=5)
    ax2.set_xlim(0, 1.1 * Renv_km)
    ax2.set_ylim(0.98 * Pmin_mb, 1020)
    ax2.set_xlabel('radius [km]')
    ax2.set_ylabel('sea-level pressure [mb]')
    ax2.set_title('Pressure profile', fontsize=12)

    ann2 = f"Pmin = {Pmin_mb:.1f} mb\nPenv = {Penv_mb:.1f} mb"
    ax2.text(
        0.1, 0.2, ann2,
        transform=ax2.transAxes,
        ha='left', va='top',
        fontsize=10,
        bbox=dict(facecolor='white', edgecolor='none', alpha=0.7)
    )

    plt.tight_layout()
    if save_path:
        fig.savefig(f"{save_path}.jpg", dpi=300)
    plt.show()

    return fig, (ax1, ax2)



# test_windprofile_all.py

## Create a fast and robust radial profile of the tropical cyclone rotating wind from inputs Vmax, R34kt, latitude, and Vtrans.


import os
import sys

# Add parent directory to the Python path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import numpy as np
import math

############################################################
# NHC/Best Track Operational Inputs
VmaxNHC_kt = 100  # [kt]; NHC storm intensity (point-max wind speed)
Vtrans_kt = 20    # [kt]; storm translation speed, usually estimated from adjacent track points; used to estimate azimuthal-mean Vmax (Vmaxmean_ms = VmaxNHC_ms - 0.55*Vtrans_ms)
lat = 20  # [degN]; default 20N; storm-center latitude
R34ktNHCquadmax_nautmi = (135 + 150 + 145 + 150) / 4 # average NHC R34kt radius (here 4 quadrants)
                                                        # these are officially the MAXIMUM radii of this wind speed in each quadrant;
                                                        # value is reduced by 0.85 within the code to estimate the mean radius (see Chavas Knaff Klotzbach 2025 for more info)
Penv_mb = 1008      #[mb]; environmental pressure, to create full pressure profile
## Default values: VmaxNHC_kt=100 kt, R34ktNHCquadmax_nautmi= 145.0 naut mi, lat = 20 --> unadjusted Rmax=38.1 km (sanity check)
############################################################


################################################################
## Calculate wind and pressure profiles and associated data
"""
Full modeling pipeline:
- Estimate Rmax from R34kt: ref Chavas and Knaff 2022 WAF)
- Estimate R0 from R34kt: approximate version of outer model ref Emanuel 2004 / Chavas et al. 2015 JAS / Chavas and Lin 2016 JAS
- Generate wind profile: merge simple inner + outer models, ref Klotzbach et al. 2022 JGR-A / Chavas and Lin 2016 JAS
- Estimate Pmin: ref Chavas Knaff Klotzbach 2025 WAF
- Generate pressure profile that matches Pmin: ref Chavas Knaff Klotzbach 2025 WAF
"""
from tcwindprofile.windprofile_all import run_full_wind_model

tc_wind_and_pressure_profile = run_full_wind_model(
    VmaxNHC_kt=VmaxNHC_kt,
    Vtrans_kt=Vtrans_kt,
    R34kt_quad_max_nautmi=R34ktNHCquadmax_nautmi,
    lat=lat,
    Penv_mb=Penv_mb,
    plot=True
)

print(f"Rmax = {tc_wind_and_pressure_profile['Rmax_km']:.1f} km")
print(f"R0 = {tc_wind_and_pressure_profile['R0_km']:.1f} km")
print(f"Pmin = {tc_wind_and_pressure_profile['Pmin_mb']:.1f} hPa")
################################################################

################################################################
## Plot that data
from tcwindprofile.plot_windprofile import plot_wind_and_pressure

# unpack
rr_km   = tc_wind_and_pressure_profile['rr_km']
vv_ms   = tc_wind_and_pressure_profile['vv_ms']
pp_mb   = tc_wind_and_pressure_profile['pp_mb']
Vmaxmean_ms   = tc_wind_and_pressure_profile['Vmaxmean_ms']
Rmax_km   = tc_wind_and_pressure_profile['Rmax_km']
V34kt_ms   = tc_wind_and_pressure_profile['V34kt_ms']
R34ktmean_km   = tc_wind_and_pressure_profile['R34ktmean_km']
R0_km   = tc_wind_and_pressure_profile['R0_km']
lat   = tc_wind_and_pressure_profile['lat']
Penv_mb   = tc_wind_and_pressure_profile['Penv_mb']
Pmin_mb   = tc_wind_and_pressure_profile['Pmin_mb']


# then:
Renv_km = R0_km
plot_wind_and_pressure(
    rr_km, vv_ms,
    Rmax_km, Vmaxmean_ms,
    R34ktmean_km, V34kt_ms,
    R0_km, lat,
    rr_km, pp_mb,
    Renv_km, Penv_mb, Pmin_mb,
    save_path='tc_wind_pressure_profiles'
)
################################################################